@@ -224,7 +224,8 @@ set_text (DesktopIcon *dicon, char *text)
 					DESKTOP_SNAP_X,
 					DESKTOP_ICON_FONT,
 					text,
-					TRUE);
+					TRUE,
+					FALSE);
 
 	gtk_signal_handler_unblock (GTK_OBJECT (dicon->text), dicon->w_changed_id);
 	gtk_signal_handler_unblock (GTK_OBJECT (dicon->text), dicon->h_changed_id);
@@ -373,7 +374,7 @@ desktop_icon_reshape (DesktopIcon *dicon)
 	args[1].name = "height";
 	gtk_object_getv (GTK_OBJECT (dicon->icon), 2, args);
 	icon_width = GTK_VALUE_DOUBLE (args[0]);
-	icon_height = GTK_VALUE_DOUBLE (args[0]);
+	icon_height = GTK_VALUE_DOUBLE (args[1]);
 
 	/* Get size of icon text */
 
