@@ -11,23 +11,10 @@
  *
  * - Put an InputOnly window over icons to be able to select them even if the user clicks on
  *   the transparent area.
- *
- * - DnD from file windows to icons.
- *
- * - DnD from icons to desktop (move icon).
- *
- * - DnD from icons to windows.
- *
- * - DnD from icons to icons (file->directory or file->executable).
- *
- * - Popup menus for icons.
- *
- * - Select icons with rubberband on the root window.
- *
  */
 
 #include <config.h>
-#include "fs.h"
+#include "global.h"
 #include <gdk/gdkx.h>
 #include <gtk/gtkinvisible.h>
 #include <gnome.h>
@@ -84,19 +71,21 @@ static DesktopIconInfo *last_selected_icon;
 /* Drag and drop sources and targets */
 
 static GtkTargetEntry dnd_icon_sources[] = {
-	{ "application/x-mc-desktop-icon", 0, TARGET_MC_DESKTOP_ICON },
-	{ "text/uri-list", 0, TARGET_URI_LIST },
-	{ "text/plain", 0, TARGET_TEXT_PLAIN },
-	{ "_NETSCAPE_URL", 0, TARGET_URL }
+	{ TARGET_MC_DESKTOP_ICON_TYPE, 0, TARGET_MC_DESKTOP_ICON },
+	{ TARGET_URI_LIST_TYPE, 0, TARGET_URI_LIST },
+	{ TARGET_TEXT_PLAIN_TYPE, 0, TARGET_TEXT_PLAIN },
+	{ TARGET_URL_TYPE, 0, TARGET_URL }
 };
 
 static GtkTargetEntry dnd_icon_targets[] = {
-	{ "text/uri-list", 0, TARGET_URI_LIST }
+	{ TARGET_MC_DESKTOP_ICON_TYPE, 0, TARGET_MC_DESKTOP_ICON },
+	{ TARGET_URI_LIST_TYPE, 0, TARGET_URI_LIST },
 };
 
 static GtkTargetEntry dnd_desktop_targets[] = {
-	{ "application/x-mc-desktop-icon", 0, TARGET_MC_DESKTOP_ICON },
-	{ "text/uri-list", 0, TARGET_URI_LIST }
+	{ "_NETSCAPE_URL", 0, TARGET_URL },
+	{ TARGET_MC_DESKTOP_ICON_TYPE, 0, TARGET_MC_DESKTOP_ICON },
+	{ TARGET_URI_LIST_TYPE, 0, TARGET_URI_LIST }
 };
 
 static int dnd_icon_nsources = sizeof (dnd_icon_sources) / sizeof (dnd_icon_sources[0]);
@@ -110,8 +99,11 @@ static GtkWidget *dnd_proxy_window;
 static int dnd_press_x, dnd_press_y;
 
 /* Whether a call to select_icon() is pending because the initial click on an
- * icon had the GDK_CONTROL_MASK in it.  */
+ * icon needs to be resolved at button release time.  Also, we store the
+ * event->state.
+ */
 static int dnd_select_icon_pending;
+static guint dnd_select_icon_pending_state;
 
 /* Whether the button release signal on a desktop icon should be stopped due to
  * the icon being selected by clicking on the text item.
@@ -137,7 +129,8 @@ static int click_current_y;
 static int click_dragging;
 
 
-static DesktopIconInfo *desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos);
+static DesktopIconInfo *desktop_icon_info_new (char *filename, char *url,
+					       int auto_pos, int xpos, int ypos);
 
 
 /* Looks for a free slot in the layout_slots array and returns the coordinates that coorespond to
@@ -333,9 +326,16 @@ icon_exists_in_list (GList *list, char *filename)
 	return NULL;
 }
 
-/* Reloads the desktop icons efficiently.  If there are "new" files for which no
- * icons have been created, then icons for them will be created started at the
- * specified position -- this is used when dragging new icons to the desktop.
+typedef struct {
+	char *filename;
+	char *url;
+} file_and_url_t;
+
+/*
+ * Reloads the desktop icons efficiently.  If there are "new" files
+ * for which no icons have been created, then icons for them will be
+ * created started at the specified position -- this is used when
+ * dragging new icons to the desktop.
  */
 static void
 reload_desktop_icons (int xpos, int ypos)
@@ -343,11 +343,11 @@ reload_desktop_icons (int xpos, int ypos)
 	struct dirent *dirent;
 	DIR *dir;
 	char *full_name;
-	int have_pos, x, y;
+	int have_pos, x, y, size;
 	DesktopIconInfo *dii;
 	GSList *need_position_list, *sl;
 	GList *all_icons, *l;
-
+	char *desktop_url;
 	
 	dir = mc_opendir (desktop_directory);
 	if (!dir) {
@@ -386,14 +386,26 @@ reload_desktop_icons (int xpos, int ypos)
 		full_name = g_concat_dir_and_file (desktop_directory, dirent->d_name);
 		have_pos = gmeta_get_icon_pos (full_name, &x, &y);
 
+		if (!gnome_metadata_get (full_name, "desktop-url", &size, &desktop_url))
+			desktop_url = NULL;
+		
 		if (have_pos) {
-			dii = desktop_icon_info_new (dirent->d_name, FALSE, x, y);
+			dii = desktop_icon_info_new (dirent->d_name, desktop_url, FALSE, x, y);
 			gtk_widget_show (dii->dicon);
 
 			g_free (full_name);
-		} else
-			need_position_list = g_slist_prepend (need_position_list,
-							      g_strdup (dirent->d_name));
+		} else {
+			file_and_url_t *fau;
+
+			fau = g_new (file_and_url_t, 1);
+			fau->filename = g_strdup (dirent->d_name);
+			fau->url = g_strdup (desktop_url);
+			
+			need_position_list = g_slist_prepend (need_position_list, fau);
+		}
+
+		if (desktop_url)
+			g_free (desktop_url);
 	}
 
 	mc_closedir (dir);
@@ -414,8 +426,6 @@ reload_desktop_icons (int xpos, int ypos)
 
 	g_list_free (all_icons);
 
-	gnome_metadata_unlock ();
-	
 	/* Now create the icons for all the files that did not have their
 	 * position set.  This makes auto-placement work correctly without
 	 * overlapping icons.
@@ -424,13 +434,20 @@ reload_desktop_icons (int xpos, int ypos)
 	need_position_list = g_slist_reverse (need_position_list);
 
 	for (sl = need_position_list; sl; sl = sl->next) {
-		dii = desktop_icon_info_new (sl->data, TRUE, xpos, ypos);
+		file_and_url_t *fau = sl->data;
+		
+		dii = desktop_icon_info_new (fau->filename, fau->url, TRUE, xpos, ypos);
 		gtk_widget_show (dii->dicon);
-		g_free (sl->data);
+
+		g_free (fau->url);
+		g_free (fau->filename);
+		g_free (fau);
 	}
 
 	g_slist_free (need_position_list);
 
+	gnome_metadata_unlock ();
+
 	/* Flush events to make the icons paint themselves */
 	x_flush_events ();
 }
@@ -645,6 +662,24 @@ text_changed (GnomeIconTextItem *iti, gpointer data)
 	return retval;
 }
 
+/*
+ * Callback when an icon's text changes and the icon reprensents an
+ * URL
+ */
+static int
+text_changed_url (GnomeIconTextItem *iti, gpointer data)
+{
+	DesktopIconInfo *dii = data;
+	char *fullname;
+	char *new_text;
+	
+	fullname = g_concat_dir_and_file (desktop_directory, dii->filename);
+	new_text = gnome_icon_text_item_get_text (iti);
+	gnome_metadata_set (fullname, "desktop-url", strlen (new_text)+1, new_text);
+
+	return TRUE;
+}
+
 /* Sets up the mouse grab for when a desktop icon is being edited */
 static void
 setup_editing_grab (DesktopIconInfo *dii)
@@ -685,6 +720,7 @@ editing_started (GnomeIconTextItem *iti, gpointer data)
 	unselect_all (dii);
 
 	gtk_grab_add (dii->dicon);
+	setup_editing_grab (dii);
 	gdk_keyboard_grab (GTK_LAYOUT (DESKTOP_ICON (dii->dicon)->canvas)->bin_window,
 			   FALSE, GDK_CURRENT_TIME);
 }
@@ -745,6 +781,10 @@ desktop_icon_info_open (DesktopIconInfo *dii)
 	char *filename;
 	file_entry *fe;
 
+	if (dii->url){
+		gnome_url_show (dii->url);
+		return;
+	}
 	filename = g_concat_dir_and_file (desktop_directory, dii->filename);
 
 	fe = file_entry_from_file (filename);
@@ -866,10 +906,13 @@ icon_event (GnomeCanvasItem *item, GdkEvent *event, gpointer data)
 				    && (!dii->selected || (event->button.state & GDK_SHIFT_MASK)))
 					icon_select_on_text = TRUE;
 
-				if ((event->button.state & GDK_CONTROL_MASK)
-				    && !(event->button.state & GDK_SHIFT_MASK))
+				if ((dii->selected
+				     && !(event->button.state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)))
+				    || ((event->button.state & GDK_CONTROL_MASK)
+					&& !(event->button.state & GDK_SHIFT_MASK))) {
 					dnd_select_icon_pending = TRUE;
-				else
+					dnd_select_icon_pending_state = event->button.state;
+				} else
 					select_icon (dii, event->button.state);
 
 				retval = TRUE;
@@ -906,8 +949,9 @@ icon_event (GnomeCanvasItem *item, GdkEvent *event, gpointer data)
 		}
 
 		if (dnd_select_icon_pending) {
-			select_icon (dii, GDK_CONTROL_MASK);
+			select_icon (dii, dnd_select_icon_pending_state);
 			dnd_select_icon_pending = FALSE;
+			dnd_select_icon_pending_state = 0;
 			retval = TRUE;
 		}
 		break;
@@ -968,9 +1012,10 @@ drag_begin (GtkWidget *widget, GdkDragContext *context, gpointer data)
 
 	if (dnd_select_icon_pending) {
 		if (!dii->selected)
-			select_icon (dii, GDK_CONTROL_MASK);
+			select_icon (dii, dnd_select_icon_pending_state);
 
 		dnd_select_icon_pending = FALSE;
+		dnd_select_icon_pending_state = 0;
 	}
 
 	/* FIXME: see if it is more than one icon and if so, use a multiple-files icon. */
@@ -1111,6 +1156,132 @@ setup_icon_dnd_source (DesktopIconInfo *dii)
 			    dii);
 }
 
+/*
+ * Callback used when we get a drag_motion event from a desktop icon.  We have
+ * to decide which operation to perform based on the type of the data the user
+ * is dragging.
+ */
+static gboolean
+icon_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gint y, guint time,
+		  gpointer data)
+{
+	DesktopIconInfo *dii;
+	char *filename;
+	file_entry *fe;
+	GdkDragAction action;
+
+	dii = data;
+
+	filename = g_concat_dir_and_file (desktop_directory, dii->filename);
+	fe = file_entry_from_file (filename);
+	g_free (filename);
+
+	action = 0; /* be pessimistic by defaulting to nothing */
+
+	if (dii->selected
+	    && gdnd_drag_context_has_target (context, TARGET_MC_DESKTOP_ICON)
+	    && (context->actions & GDK_ACTION_MOVE))
+		action = GDK_ACTION_MOVE;
+	else if (gdnd_drag_context_has_target (context, TARGET_URI_LIST)) {
+		if (S_ISDIR (fe->buf.st_mode) || fe->f.link_to_dir)
+			action = context->suggested_action;
+		else if (is_exe (fe->buf.st_mode)
+			 && if_link_is_exe (fe)
+			 && (context->actions & GDK_ACTION_COPY))
+			action = GDK_ACTION_COPY;
+	}
+
+	gdk_drag_status (context, action, time);
+	return TRUE;
+}
+
+/*
+ * Returns the desktop icon that started the drag from the specified
+ * context
+ */
+static DesktopIconInfo *
+find_icon_by_drag_context (GdkDragContext *context)
+{
+	GtkWidget *source;
+	int i;
+	GList *l;
+	DesktopIconInfo *dii;
+
+	source = gtk_drag_get_source_widget (context);
+	if (!source)
+		return NULL;
+
+	source = gtk_widget_get_toplevel (source);
+
+	for (i = 0; i < (layout_cols * layout_rows); i++)
+		for (l = layout_slots[i].icons; l; l = l->next) {
+			dii = l->data;
+
+			if (dii->dicon == source)
+				return dii;
+		}
+
+	return NULL;
+}
+
+/*
+ * Performs a drop of desktop icons onto the desktop.  It basically
+ * moves the icons from their original position to the new
+ * coordinates.
+ */
+static void
+drop_desktop_icons (GdkDragContext *context, GtkSelectionData *data, int x, int y)
+{
+	DesktopIconInfo *source_dii, *dii;
+	int dx, dy;
+	int i;
+	GList *l;
+	GSList *sel_icons, *sl;
+
+	/*
+	 * FIXME: this needs to do the right thing (what Windows does)
+	 * when desktop_auto_placement is enabled.
+	 */
+
+	/* Find the icon that the user is dragging */
+
+	source_dii = find_icon_by_drag_context (context);
+	if (!source_dii) {
+		g_warning ("Eeeeek, could not find the icon that started the drag!");
+		return;
+	}
+
+	/* Compute the distance to move icons */
+
+	if (desktop_snap_icons)
+		get_icon_snap_pos (&x, &y);
+
+	dx = x - source_dii->x - dnd_press_x;
+	dy = y - source_dii->y - dnd_press_y;
+
+	/* Build a list of selected icons */
+
+	sel_icons = NULL;
+
+	for (i = 0; i < (layout_cols * layout_rows); i++)
+		for (l = layout_slots[i].icons; l; l = l->next) {
+			dii = l->data;
+			if (dii->selected)
+				sel_icons = g_slist_prepend (sel_icons, l->data);
+		}
+
+	/* Move the icons */
+
+	for (sl = sel_icons; sl; sl = sl->next) {
+		dii = sl->data;
+		desktop_icon_info_place (dii, FALSE, dii->x + dx, dii->y + dy);
+	}
+
+	/* Clean up */
+
+	g_slist_free (sel_icons);
+}
+
 /**
  * drop_on_file_entry
  */
@@ -1120,7 +1291,8 @@ desktop_icon_drop_uri_list (DesktopIconInfo *dii, GdkDragContext *context, GtkSe
 	char *filename;
 	file_entry *fe;
 	int size;
-	char *buf, *mime_type;
+	char *buf;
+	const char *mime_type;
 	
 	filename = g_concat_dir_and_file (desktop_directory, dii->filename);
 
@@ -1209,7 +1381,16 @@ icon_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gin
 	dii = user_data;
 
 	switch (info) {
+	case TARGET_MC_DESKTOP_ICON:
+		if (dii->selected)
+			drop_desktop_icons (context, data, x + dii->x, y + dii->y);
+		else
+			printf ("FIXME: what do drop?\n"); /* FIXME */
+
+		break;
+
 	case TARGET_URI_LIST:
+		printf ("Wheeeeee!\n");
 		desktop_icon_drop_uri_list (dii, context, data);
 		break;
 
@@ -1222,41 +1403,17 @@ icon_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gin
 static void
 setup_icon_dnd_dest (DesktopIconInfo *dii)
 {
-	char *filename;
-	file_entry *fe;
-	int actions;
-
-	filename = g_concat_dir_and_file (desktop_directory, dii->filename);
-	fe = file_entry_from_file (filename);
-	g_free (filename);
-
-	if (!fe)
-		return; /* eek */
-
-	/* See what actions are appropriate for this icon */
-
-	if (fe->f.link_to_dir)
-		actions = GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK;
-	else if (is_exe (fe->buf.st_mode) && if_link_is_exe (fe))
-		actions = GDK_ACTION_COPY;
-	else
-		actions = 0;
-
-	file_entry_free (fe);
-
-	if (!actions)
-		return;
-
-	/* Connect the drop signals */
-
 	gtk_drag_dest_set (DESKTOP_ICON (dii->dicon)->canvas,
-			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+			   GTK_DEST_DEFAULT_DROP,
 			   dnd_icon_targets,
 			   dnd_icon_ntargets,
-			   actions);
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 
+	gtk_signal_connect (GTK_OBJECT (DESKTOP_ICON (dii->dicon)->canvas), "drag_motion",
+			    (GtkSignalFunc) (icon_drag_motion),
+			    dii);
 	gtk_signal_connect (GTK_OBJECT (DESKTOP_ICON (dii->dicon)->canvas), "drag_data_received",
-			    GTK_SIGNAL_FUNC (icon_drag_data_received),
+			    (GtkSignalFunc) icon_drag_data_received,
 			    dii);
 }
 
@@ -1268,18 +1425,19 @@ setup_icon_dnd_dest (DesktopIconInfo *dii)
  * not show the icon.
  */
 static DesktopIconInfo *
-desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos)
+desktop_icon_info_new (char *filename, char *url, int auto_pos, int xpos, int ypos)
 {
 	DesktopIconInfo *dii;
 	file_entry *fe;
 	char *full_name;
 	GdkImlibImage *icon_im;
-
+	GtkSignalFunc text_changed_func;
+	
 	/* Create the icon structure */
 
 	full_name = g_concat_dir_and_file (desktop_directory, filename);
 	fe = file_entry_from_file (full_name);
-	icon_im = gicon_get_icon_for_file_speed (fe, FALSE);
+	icon_im = gicon_get_icon_for_file_speed (desktop_directory, fe, FALSE);
 
 	dii = g_new (DesktopIconInfo, 1);
 	dii->dicon = desktop_icon_new (icon_im, filename);
@@ -1288,7 +1446,8 @@ desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos)
 	dii->slot = -1;
 	dii->filename = g_strdup (filename);
 	dii->selected = FALSE;
-
+	dii->url = g_strdup (url);
+	
 	file_entry_free (fe);
 	g_free (full_name);
 
@@ -1318,9 +1477,14 @@ desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos)
 
 	/* Connect to the text item's signals */
 
+	if (dii->url)
+		text_changed_func = (GtkSignalFunc) text_changed_url;
+	else
+		text_changed_func = (GtkSignalFunc) text_changed;
+
 	gtk_signal_connect (GTK_OBJECT (DESKTOP_ICON (dii->dicon)->text), "text_changed",
-			    (GtkSignalFunc) text_changed,
-			    dii);
+			    text_changed_func, dii);
+
 	gtk_signal_connect (GTK_OBJECT (DESKTOP_ICON (dii->dicon)->text), "editing_started",
 			    (GtkSignalFunc) editing_started,
 			    dii);
@@ -1354,6 +1518,7 @@ desktop_icon_info_destroy (DesktopIconInfo *dii)
 	gtk_widget_destroy (dii->dicon);
 	remove_from_slot (dii);
 
+	g_free (dii->url);
 	g_free (dii->filename);
 	g_free (dii);
 }
@@ -1508,86 +1673,70 @@ setup_xdnd_proxy (guint32 xid, GdkWindow *proxy_window)
 		return FALSE;
 }
 
-/* Returns the desktop icon that started the drag from the specified context */
-static DesktopIconInfo *
-find_icon_by_drag_context (GdkDragContext *context)
+/* Callback used when we get a drag_motion event from the desktop.  We must
+ * decide what kind of operation can be performed with what the user is
+ * dragging.
+ */
+static gboolean
+desktop_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gint y, guint time,
+		     gpointer data)
 {
-	GtkWidget *source;
-	int i;
-	GList *l;
-	DesktopIconInfo *dii;
+	GdkDragAction action;
+	GtkWidget *source_widget;
 
-	source = gtk_drag_get_source_widget (context);
-	if (!source)
-		return NULL;
+	action = context->suggested_action; /* this is the default */
 
-	source = gtk_widget_get_toplevel (source);
+	if (gdnd_drag_context_has_target (context, TARGET_MC_DESKTOP_ICON)) {
+		if (context->actions & GDK_ACTION_MOVE)
+			action = GDK_ACTION_MOVE;
+	} else if (gdnd_drag_context_has_target (context, TARGET_URI_LIST)) {
+		source_widget = gtk_drag_get_source_widget (context);
 
-	for (i = 0; i < (layout_cols * layout_rows); i++)
-		for (l = layout_slots[i].icons; l; l = l->next) {
-			dii = l->data;
+		/* If it comes from ourselves, make move the default unless the
+		 * user is explicitly asking for ASK.
+		 */
+		printf ("%s\t%s\t%s\t%s\n",
+			(context->actions & GDK_ACTION_COPY) ? "copy" : "",
+			(context->actions & GDK_ACTION_MOVE) ? "move" : "",
+			(context->actions & GDK_ACTION_LINK) ? "link" : "",
+			(context->actions & GDK_ACTION_ASK) ? "ask" : "");
+			
+		if (source_widget
+		    && context->suggested_action != GDK_ACTION_ASK
+		    && (context->actions & GDK_ACTION_MOVE))
+			action = GDK_ACTION_MOVE;
+	} else
+		action = 0; /* we cannot handle that type of data */
 
-			if (dii->dicon == source)
-				return dii;
-		}
-
-	return NULL;
+	gdk_drag_status (context, action, time);
+	return TRUE;
 }
 
 /*
- * Performs a drop of desktop icons onto the desktop.  It basically moves the icons from their
- * original position to the new coordinates.
+ * drop_url: Invoked when we have received an URL from Netscape.
+ * Install the url on the ~/desktop directory
  */
 static void
-drop_desktop_icons (GdkDragContext *context, GtkSelectionData *data, int x, int y)
+drop_url (GdkDragContext *context, GtkSelectionData *data, gint x, gint y)
 {
-	DesktopIconInfo *source_dii, *dii;
-	int dx, dy;
-	int i;
-	GList *l;
-	GSList *sel_icons, *sl;
+	char *template;
 
-	/* FIXME: this needs to do the right thing (what Windows does) when desktop_auto_placement
-	 * is enabled.
-	 */
+	template = g_concat_dir_and_file (desktop_directory, "urlXXXXXX");
 
-	/* Find the icon that the user is dragging */
+	if (mktemp (template)){
+		FILE *f;
 
-	source_dii = find_icon_by_drag_context (context);
-	if (!source_dii) {
-		g_warning ("Eeeeek, could not find the icon that started the drag!");
-		return;
-	}
+		f = fopen (template, "w");
+		if (f){
+			fprintf (f, "URL: %s\n", data->data);
+			fclose (f);
 
-	/* Compute the distance to move icons */
-
-	if (desktop_snap_icons)
-		get_icon_snap_pos (&x, &y);
-
-	dx = x - source_dii->x - dnd_press_x;
-	dy = y - source_dii->y - dnd_press_y;
-
-	/* Build a list of selected icons */
-
-	sel_icons = NULL;
-
-	for (i = 0; i < (layout_cols * layout_rows); i++)
-		for (l = layout_slots[i].icons; l; l = l->next) {
-			dii = l->data;
-			if (dii->selected)
-				sel_icons = g_slist_prepend (sel_icons, l->data);
+			gnome_metadata_set (template, "desktop-url",
+					    strlen (data->data)+1, data->data);
 		}
-
-	/* Move the icons */
-
-	for (sl = sel_icons; sl; sl = sl->next) {
-		dii = sl->data;
-		desktop_icon_info_place (dii, FALSE, dii->x + dx, dii->y + dy);
 	}
 
-	/* Clean up */
-
-	g_slist_free (sel_icons);
+	g_free (template);
 }
 
 /* Callback used when the root window receives a drop */
@@ -1604,27 +1753,23 @@ desktop_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x,
 	x += dx;
 	y += dy;
 
+	printf ("Drag!!\n");
 	switch (info) {
 	case TARGET_MC_DESKTOP_ICON:
 		drop_desktop_icons (context, data, x, y);
 		break;
 
 	case TARGET_URI_LIST:
-		/*
-		 * Unless the user is dragging with button-2 (ask action)
-		 * drops on the desktop will be symlinks.
-		 *
-		 * I have got enough complaints as it is.
-		 */
-		if (context->suggested_action != GDK_ACTION_ASK)
-			context->suggested_action = GDK_ACTION_LINK;
-				
 		retval = gdnd_drop_on_directory (context, data, desktop_directory);
 		if (retval)
 			reload_desktop_icons (x, y);
-
 		break;
 
+	case TARGET_URL:
+		printf ("Aqui!\n");
+		drop_url (context, data, x, y);
+		break;
+		
 	default:
 		break;
 	}
@@ -1641,13 +1786,16 @@ setup_desktop_dnd (void)
 		g_warning ("There is already a process taking drop windows on the desktop\n");
 
 	gtk_drag_dest_set (dnd_proxy_window,
-			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+			   GTK_DEST_DEFAULT_DROP,
 			   dnd_desktop_targets,
 			   dnd_desktop_ntargets,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 
+	gtk_signal_connect (GTK_OBJECT (dnd_proxy_window), "drag_motion",
+			    (GtkSignalFunc) desktop_drag_motion,
+			    NULL);
 	gtk_signal_connect (GTK_OBJECT (dnd_proxy_window), "drag_data_received",
-			    GTK_SIGNAL_FUNC (desktop_drag_data_received),
+			    (GtkSignalFunc) desktop_drag_data_received,
 			    NULL);
 }
 
@@ -2139,6 +2287,7 @@ setup_desktop_clicks (void)
 void
 desktop_init (void)
 {
+	gdnd_init ();
 	create_layout_info ();
 	create_desktop_dir ();
 	reload_desktop_icons (0, 0);
