@@ -10,10 +10,8 @@
 
 #include <config.h>
 #include <string.h>
-#include <stdlib.h>		/* atoi */
-#include "fs.h"
-#include "mad.h"
 #include "x.h"
+#include "global.h"
 #include "dir.h"
 #include "command.h"
 #include "panel.h"
@@ -27,13 +25,13 @@
 #include "gscreen.h"
 #include "dir.h"
 #include "dialog.h"
+#include "setup.h"
 #include "gdesktop.h"
 #include "gdnd.h"
 #include "gtkdtree.h"
 #include "gpageprop.h"
 #include "gpopup.h"
 #include "gcliplabel.h"
-#include "gblist.h"
 #include "gicon.h"
 #include "../vfs/vfs.h"
 #include <gdk/gdkprivate.h>
@@ -42,7 +40,7 @@
 int tree_panel_visible = -1;
 
 /* The pixmaps */
-#include "directory.xpm"
+#include "dir-close.xpm"
 #include "link.xpm"
 #include "dev.xpm"
 #include "listing-list.xpm"
@@ -61,13 +59,13 @@ GdkPixmap *icon_dev_pixmap;
 GdkBitmap *icon_dev_mask;
 
 static GtkTargetEntry drag_types [] = {
-	{ "text/uri-list", 0, TARGET_URI_LIST },
-	{ "text/plain",    0, TARGET_TEXT_PLAIN },
-	{ "_NETSCAPE_URL", 0, TARGET_URL }
+	{ TARGET_URI_LIST_TYPE, 0, TARGET_URI_LIST },
+	{ TARGET_TEXT_PLAIN_TYPE, 0, TARGET_TEXT_PLAIN },
+	{ TARGET_URL_TYPE, 0, TARGET_URL }
 };
 
 static GtkTargetEntry drop_types [] = {
-	{ "text/uri-list", 0, TARGET_URI_LIST }
+	{ TARGET_URI_LIST_TYPE, 0, TARGET_URI_LIST }
 };
 
 #define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
@@ -170,7 +168,7 @@ panel_fill_panel_list (WPanel *panel)
 	int i, col, type_col, color;
 	char  **texts;
 
-	texts = malloc (sizeof (char *) * (items+1));
+	texts = g_new (char *, items+1);
 
 	gtk_clist_freeze (GTK_CLIST (cl));
 	gtk_clist_clear (GTK_CLIST (cl));
@@ -201,16 +199,21 @@ panel_fill_panel_list (WPanel *panel)
 		}
 		gtk_clist_append (cl, texts);
 
-		color = file_compute_color (fe->f.marked ? MARKED : NORMAL, fe);
+		color = file_compute_color (NORMAL, fe);
 		panel_file_list_set_row_colors (cl, i, color);
+
 		if (type_col != -1)
 			panel_file_list_set_type_bitmap (cl, i, type_col, color, fe);
+
+		if (fe->f.marked)
+			gtk_clist_select_row (cl, i, 0);
+		
 	}
 	/* This is needed as the gtk_clist_append changes selected under us :-( */
 	panel->selected = selected;
 	select_item (panel);
 	gtk_clist_thaw (GTK_CLIST (cl));
-	free (texts);
+	 g_free (texts);
 }
 
 /*
@@ -232,7 +235,7 @@ panel_fill_panel_icons (WPanel *panel)
 		file_entry *fe = &panel->dir.list [i];
 		int p;
 		
-		image = gicon_get_icon_for_file (fe);
+		image = gicon_get_icon_for_file (panel->cwd, fe);
 		p = gnome_icon_list_append_imlib (icons, image, fe->fname);
 		if (fe->f.marked)
 			gnome_icon_list_select_icon (icons, p);
@@ -301,18 +304,19 @@ x_panel_select_item (WPanel *panel, int index, int value)
 {
 	int color;
 
-	color = file_compute_color (value ? MARKED : NORMAL, &panel->dir.list[index]);
+	color = file_compute_color (NORMAL, &panel->dir.list[index]);
 	panel_file_list_set_row_colors (CLIST_FROM_SW (panel->list), index, color);
 }
 
 void
 x_select_item (WPanel *panel)
 {
+	do_file_mark (panel, panel->selected, 1);
+	display_mini_info (panel);
+	
 	if (panel->list_type == list_icons){
 		GnomeIconList *list = GNOME_ICON_LIST (panel->icons);
 
-		do_file_mark (panel, panel->selected, 1);
-		display_mini_info (panel);
 		gnome_icon_list_select_icon (list, panel->selected);
 
 		if (list->icon_list){
@@ -323,15 +327,8 @@ x_select_item (WPanel *panel)
 		gnome_canvas_update_now (GNOME_CANVAS (list));
 	} else {
 		GtkCList *clist = CLIST_FROM_SW (panel->list);
-		int color, marked;
 
-		if (panel->dir.list [panel->selected].f.marked)
-			marked = 1;
-		else
-			marked = 0;
-
-		color = file_compute_color (marked ? MARKED_SELECTED : SELECTED, &panel->dir.list [panel->selected]);
-		panel_file_list_set_row_colors (CLIST_FROM_SW (panel->list), panel->selected, color);
+		gtk_clist_select_row (clist, panel->selected, 0);
 
 		/* Make it visible */
 		if (gtk_clist_row_is_visible (clist, panel->selected) != GTK_VISIBILITY_FULL)
@@ -342,20 +339,14 @@ x_select_item (WPanel *panel)
 void
 x_unselect_item (WPanel *panel)
 {
-	if (panel->list_type == list_icons){
-	        int selected = panel->selected;
-
-		/* This changes the panel->selected */
+	int selected = panel->selected;
+	
+	if (panel->list_type == list_icons)
 		gnome_icon_list_unselect_all (GNOME_ICON_LIST (panel->icons), NULL, NULL);
-		panel->selected = selected;
-	} else {
-		int color;
-		int val;
+	else
+		gtk_clist_unselect_all (CLIST_FROM_SW (panel->list));
 
-		val = panel->dir.list [panel->selected].f.marked ? MARKED : NORMAL;
-		color = file_compute_color (val, &panel->dir.list [panel->selected]);
-		panel_file_list_set_row_colors (CLIST_FROM_SW (panel->list), panel->selected, color);
-	}
+	panel->selected = selected;
 }
 
 void
@@ -465,14 +456,6 @@ panel_file_list_configure_contents (GtkWidget *sw, WPanel *panel, int main_width
 	gtk_clist_thaw (clist);
 }
 
-static void
-internal_select_item (GtkWidget *file_list, WPanel *panel, int row)
-{
-	unselect_item (panel);
-	panel->selected = row;
-
-	select_item (panel);
-}
 static int
 panel_file_list_press_row (GtkWidget *file_list, GdkEvent *event, WPanel *panel)
 {
@@ -489,43 +472,35 @@ panel_file_list_press_row (GtkWidget *file_list, GdkEvent *event, WPanel *panel)
 static void
 panel_file_list_select_row (GtkWidget *file_list, int row, int column, GdkEvent *event, WPanel *panel)
 {
-	int current_selection = panel->selected;
-	char *fullname;
-	
-	if (!event) {
-		internal_select_item (file_list, panel, row);
+	panel->selected = row;
+	do_file_mark (panel, row, 1);
+	display_mini_info (panel);
+	execute_hooks (select_file_hook);
+
+
+	if (!event)
 		return;
-	}
-
+	
 	switch (event->type) {
-	case GDK_BUTTON_RELEASE:
-		gtk_clist_unselect_row (CLIST_FROM_SW (panel->list), row, 0);
-		internal_select_item (file_list, panel, row);
-
-		switch (event->button.button) {
-		case 1:
-			if (!(event->button.state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK))){
-				do_file_mark (panel, row, !panel->dir.list[row].f.marked);
-			} else {
-				/* if shift-click is pressed */
-				do_file_mark_range (panel, row, current_selection);
-			}
-			break;
-
-		case 2:
-			if (S_ISDIR (panel->dir.list [current_selection].buf.st_mode) ||
-			    panel->dir.list [current_selection].f.link_to_dir){
-				fullname = concat_dir_and_file (panel->cwd, panel->dir.list [current_selection].fname);
-				new_panel_at (fullname);
-				free (fullname);
-			}
-			break;
-		}
-
+	case GDK_BUTTON_PRESS:
+		if (event->button.button == 3)
+			gpopup_do_popup ((GdkEventButton *) event, panel, NULL, row, panel->dir.list[row].fname);
 		break;
+		
+	case GDK_BUTTON_RELEASE:
+		if (event->button.button == 2){
+			char *fullname;
 
+			if (S_ISDIR (panel->dir.list [row].buf.st_mode) ||
+			    panel->dir.list [row].f.link_to_dir){
+				fullname = concat_dir_and_file (panel->cwd, panel->dir.list [row].fname);
+				new_panel_at (fullname);
+				 g_free (fullname);
+			}
+		}
+		break;
+		
 	case GDK_2BUTTON_PRESS:
-		gtk_clist_unselect_row (CLIST_FROM_SW (panel->list), row, 0);
 		if (event->button.button == 1)
 			do_enter (panel);
 		break;
@@ -535,6 +510,16 @@ panel_file_list_select_row (GtkWidget *file_list, int row, int column, GdkEvent
 	}
 }
 
+static void
+panel_file_list_unselect_row (GtkWidget *widget, int row, int columns, GdkEvent *event, WPanel *panel)
+{
+	do_file_mark (panel, row, 0);
+	display_mini_info (panel);
+	if (panel->marked == 0)
+		panel->selected = 0;
+}
+
+
 /* Figure out the number of visible lines in the panel */
 static void
 panel_file_list_compute_lines (GtkScrolledWindow *sw, WPanel *panel, int height)
@@ -604,7 +589,7 @@ panel_create_pixmaps (void)
 {
 	pixmaps_ready = TRUE;
 
-	create_pixmap (directory_xpm, &icon_directory_pixmap, &icon_directory_mask);
+	create_pixmap (DIRECTORY_CLOSE_XPM, &icon_directory_pixmap, &icon_directory_mask);
 	create_pixmap (link_xpm, &icon_link_pixmap, &icon_link_mask);
 	create_pixmap (dev_xpm, &icon_dev_pixmap, &icon_dev_mask);
 }
@@ -613,7 +598,7 @@ typedef gboolean (*desirable_fn)(WPanel *p, int x, int y);
 typedef gboolean (*scroll_fn)(gpointer data);
 
 static gboolean
-panel_setup_drag_motion (WPanel *panel, int x, int y, desirable_fn desirable, scroll_fn scroll)
+panel_setup_drag_scroll (WPanel *panel, int x, int y, desirable_fn desirable, scroll_fn scroll)
 {
 	if (panel->timer_id != -1){
 		gtk_timeout_remove (panel->timer_id);
@@ -631,15 +616,6 @@ panel_setup_drag_motion (WPanel *panel, int x, int y, desirable_fn desirable, sc
 	return FALSE;
 }
 
-static void
-panel_file_list_scrolled (GtkAdjustment *adj, WPanel *panel)
-{
-	if (!GTK_IS_ADJUSTMENT (adj)) {
-		fprintf (stderr, "file_list_is_scrolled is called and there are not enough boats!\n");
-		exit (1);
-	}
-}
-
 static void
 panel_configure_file_list (WPanel *panel, GtkWidget *sw, GtkWidget *file_list)
 {
@@ -653,7 +629,7 @@ panel_configure_file_list (WPanel *panel, GtkWidget *sw, GtkWidget *file_list)
 
 	/* Configure the CList */
 
-	gtk_clist_set_selection_mode (GTK_CLIST (file_list), GTK_SELECTION_SINGLE);
+	gtk_clist_set_selection_mode (GTK_CLIST (file_list), GTK_SELECTION_EXTENDED);
 
 	for (i = 0, format = panel->format; format; format = format->next) {
 		GtkJustification just;
@@ -662,10 +638,17 @@ panel_configure_file_list (WPanel *panel, GtkWidget *sw, GtkWidget *file_list)
 			continue;
 
 		/* Set desired justification */
-		if (format->just_mode == J_LEFT)
-			just = GTK_JUSTIFY_LEFT;
-		else
-			just = GTK_JUSTIFY_RIGHT;
+		switch (HIDE_FIT(format->just_mode)){
+		case J_LEFT:
+		    just = GTK_JUSTIFY_LEFT;
+		    break;
+		case J_RIGHT:
+		    just = GTK_JUSTIFY_RIGHT;
+		    break;
+		case J_CENTER:
+		    just = GTK_JUSTIFY_CENTER;
+		    break;    
+		}
 
 		gtk_clist_set_column_justification (GTK_CLIST (file_list), i, just);
 		i++;
@@ -673,8 +656,6 @@ panel_configure_file_list (WPanel *panel, GtkWidget *sw, GtkWidget *file_list)
 
 	/* Configure the scrolbars */
 	adjustment = GTK_OBJECT (gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (sw)));
-	gtk_signal_connect_after (adjustment, "value_changed",
-				  GTK_SIGNAL_FUNC (panel_file_list_scrolled), panel);
 }
 
 /*
@@ -698,7 +679,7 @@ panel_build_selected_file_list (WPanel *panel, int *file_list_len)
 				total_len += (filelen + cwdlen + panel->dir.list [i].fnamelen + seplen);
 
 		total_len++;
-		data = copy = xmalloc (total_len+1, "build_selected_file_list");
+		data = copy = g_malloc (total_len+1);
 		for (i = 0; i < panel->count; i++)
 			if (panel->dir.list [i].f.marked){
 				strcpy (copy, "file:");
@@ -716,8 +697,8 @@ panel_build_selected_file_list (WPanel *panel, int *file_list_len)
 
 		fullname = concat_dir_and_file (panel->cwd, panel->dir.list [panel->selected].fname);
 
-		uri = copy_strings ("file:", fullname, NULL);
-		free (fullname);
+		uri = g_strconcat ("file:", fullname, NULL);
+		g_free (fullname);
 
 		*file_list_len = strlen (uri) + 1;
 		return uri;
@@ -811,7 +792,7 @@ panel_icon_list_drag_data_received (GtkWidget          *widget,
 	}
 
 	reload = gdnd_drop_on_directory (context, selection_data, dir);
-	free (dir);
+	g_free (dir);
 
 	if (reload){
 		update_one_panel_widget (panel, 0, UP_KEEPSEL);
@@ -852,7 +833,7 @@ panel_clist_drag_data_received (GtkWidget          *widget,
 	}
 
 	gdnd_drop_on_directory (context, selection_data, dir);
-	free (dir);
+	g_free (dir);
 
 	update_one_panel_widget (panel, 0, UP_KEEPSEL);
 	panel_update_contents (panel);
@@ -911,6 +892,9 @@ load_dnd_icons (void)
 static int
 panel_clist_button_press (GtkWidget *widget, GdkEventButton *event, WPanel *panel)
 {
+	if (event->window != GTK_CLIST (widget)->clist_window)
+		return FALSE;
+
 	panel->maybe_start_drag = event->button;
 
 	panel->click_x = event->x;
@@ -931,8 +915,7 @@ panel_widget_motion (GtkWidget *widget, GdkEventMotion *event, WPanel *panel)
 {
 	GtkTargetList *list;
 	GdkDragContext *context;
-	int action;
-		
+
 	if (!panel->maybe_start_drag)
 		return FALSE;
 
@@ -942,12 +925,9 @@ panel_widget_motion (GtkWidget *widget, GdkEventMotion *event, WPanel *panel)
 
 	list = gtk_target_list_new (drag_types, ELEMENTS (drag_types));
 
-	if (panel->maybe_start_drag == 2)
-		action = GDK_ACTION_ASK;
-	else
-		action = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
-	
-	context = gtk_drag_begin (widget, list, action,
+	context = gtk_drag_begin (widget, list,
+				  (GDK_ACTION_COPY | GDK_ACTION_MOVE
+				   | GDK_ACTION_LINK | GDK_ACTION_ASK),
 				  panel->maybe_start_drag, (GdkEvent *) event);
 	gtk_drag_set_icon_default (context);
 
@@ -976,6 +956,26 @@ panel_drag_end (GtkWidget *widget, GdkDragContext *context, WPanel *panel)
 	panel->dragging = 0;
 }
 
+/* Wrapper for the motion_notify callback; it ignores motion events from the
+ * clist if they do not come from the clist_window.
+ */
+static int
+panel_clist_motion (GtkWidget *widget, GdkEventMotion *event, gpointer data)
+{
+	if (event->window != GTK_CLIST (widget)->clist_window)
+		return FALSE;
+
+	panel_widget_motion (widget, event, data);
+
+	/* We have to stop the motion event from ever reaching the clist.
+	 * Otherwise it will begin dragging/selecting rows when we don't want
+	 * that.  Yes, the clist widget sucks.
+	 */
+
+	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "motion_notify_event");
+	return TRUE;
+}
+
 /**
  * panel_clist_scrolling_is_desirable:
  *
@@ -994,8 +994,8 @@ panel_clist_scrolling_is_desirable (WPanel *panel, int x, int y)
 		if (va->value > va->lower)
 			return TRUE;
 	} else {
-		if (y > (GTK_WIDGET (panel->list)->allocation.height-20)){
-			if (va->value < va->upper)
+		if (y > (GTK_WIDGET (panel->list)->allocation.height - 10)){
+			if (va->value < va->upper - va->page_size)
 				return TRUE;
 		}
 	}
@@ -1014,29 +1014,71 @@ panel_clist_scroll (gpointer data)
 {
 	WPanel *panel = data;
 	GtkAdjustment *va;
+	double v;
 
 	va = scrolled_window_get_vadjustment (panel->list);
 
-	if (panel->drag_motion_y < 10)
-		gtk_adjustment_set_value (va, va->value - va->step_increment);
-	else{
-		gtk_adjustment_set_value (va, va->value + va->step_increment);
+	if (panel->drag_motion_y < 10) {
+		v = va->value - va->step_increment;
+		if (v < va->lower)
+			v = va->lower;
+
+		gtk_adjustment_set_value (va, v);
+	} else {
+		v = va->value + va->step_increment;
+		if (v > va->upper - va->page_size)
+			v = va->upper - va->page_size;
+
+		gtk_adjustment_set_value (va, v);
 	}
+
 	return TRUE;
 }
 
-/**
- * panel_clist_drag_motion:
- *
- * Invoked when an application dragging over us has the the cursor moved.
- * If we are close to the top or bottom, we scroll the window
+/* Callback used for drag motion events over the clist.  We set up
+ * auto-scrolling and validate the drop to present the user with the correct
+ * feedback.
  */
 static gboolean
-panel_clist_drag_motion (GtkWidget *widget, GdkDragContext *ctx, int x, int y, guint time, void *data)
+panel_clist_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gint y, guint time,
+			 gpointer data)
 {
-	WPanel *panel = data;
-	
-	panel_setup_drag_motion (panel, x, y, panel_clist_scrolling_is_desirable, panel_clist_scroll);
+	WPanel *panel;
+	GdkDragAction action;
+	GtkWidget *source_widget;
+	gint idx;
+	file_entry *fe;
+
+	panel = data;
+
+	if (context->dest_window != GTK_CLIST (widget)->clist_window) {
+		printf ("squick\n");
+		gdk_drag_status (context, 0, time);
+		return TRUE;
+	}
+
+	/* Set up auto-scrolling */
+
+	panel_setup_drag_scroll (panel, x, y,
+				 panel_clist_scrolling_is_desirable,
+				 panel_clist_scroll);
+
+	/* Validate the drop */
+
+	gdnd_find_panel_by_drag_context (context, &source_widget);
+
+	if (!gtk_clist_get_selection_info (GTK_CLIST (widget), x, y, &idx, NULL))
+		fe = NULL;
+	else
+		fe = &panel->dir.list[idx];
+
+	action = gdnd_validate_action (context,
+				       source_widget != NULL,
+				       source_widget == widget,
+				       fe,
+				       fe ? fe->f.marked : FALSE);
+
+	gdk_drag_status (context, action, time);
 	return TRUE;
 }
 
@@ -1074,8 +1116,8 @@ panel_icon_list_scrolling_is_desirable (WPanel *panel, int x, int y)
 		if (va->value > va->lower)
 			return TRUE;
 	} else {
-		if (y > (GTK_WIDGET (panel->icons)->allocation.height-20)){
-			if (va->value < va->upper)
+		if (y > (GTK_WIDGET (panel->icons)->allocation.height - 10)){
+			if (va->value < va->upper - va->page_size)
 				return TRUE;
 		}
 	}
@@ -1094,28 +1136,63 @@ panel_icon_list_scroll (gpointer data)
 {
 	WPanel *panel = data;
 	GtkAdjustment *va;
+	double v;
 
 	va = GNOME_ICON_LIST (panel->icons)->adj;
 
-	if (panel->drag_motion_y < 10)
-		gtk_adjustment_set_value (va, va->value - va->step_increment);
-	else{
-		gtk_adjustment_set_value (va, va->value + va->step_increment);
+	if (panel->drag_motion_y < 10) {
+		v = va->value - va->step_increment;
+		if (v < va->lower)
+			v = va->lower;
+
+		gtk_adjustment_set_value (va, v);
+	} else {
+		v = va->value + va->step_increment;
+		if (v > va->upper - va->page_size)
+			v = va->upper - va->page_size;
+
+		gtk_adjustment_set_value (va, v);
 	}
+
 	return TRUE;
 }
-/**
- * panel_icon_list_drag_motion:
- *
- * Invoked when an application dragging over us has the the cursor moved.
- * If we are close to the top or bottom, we scroll the window
+
+/* Callback used for drag motion events in the icon list.  We need to set up
+ * auto-scrolling and validate the drop to present the user with the correct
+ * feedback.
  */
 static gboolean
-panel_icon_list_drag_motion (GtkWidget *widget, GdkDragContext *ctx, int x, int y, guint time, void *data)
+panel_icon_list_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gint y, guint time,
+			     gpointer data)
 {
-	WPanel *panel = data;
-	
-	panel_setup_drag_motion (panel, x, y, panel_icon_list_scrolling_is_desirable, panel_icon_list_scroll);
+	WPanel *panel;
+	GdkDragAction action;
+	GtkWidget *source_widget;
+	int idx;
+	file_entry *fe;
+
+	panel = data;
+
+	/* Set up auto-scrolling */
+
+	panel_setup_drag_scroll (panel, x, y,
+				 panel_icon_list_scrolling_is_desirable,
+				 panel_icon_list_scroll);
+
+	/* Validate the drop */
+
+	gdnd_find_panel_by_drag_context (context, &source_widget);
+
+	idx = gnome_icon_list_get_icon_at (GNOME_ICON_LIST (widget), x, y);
+	fe = (idx == -1) ? NULL : &panel->dir.list[idx];
+
+	action = gdnd_validate_action (context,
+				       source_widget != NULL,
+				       source_widget == widget,
+				       fe,
+				       fe ? fe->f.marked : FALSE);
+
+	gdk_drag_status (context, action, time);
 	return TRUE;
 }
 
@@ -1156,7 +1233,7 @@ panel_create_file_list (WPanel *panel)
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
-	file_list = gtk_blist_new_with_titles (items, titles);
+	file_list = gtk_clist_new_with_titles (items, titles);
 	gtk_container_add (GTK_CONTAINER (sw), file_list);
 	gtk_widget_show (file_list);
 
@@ -1170,9 +1247,14 @@ panel_create_file_list (WPanel *panel)
 	gtk_signal_connect (GTK_OBJECT (file_list), "select_row",
 			    GTK_SIGNAL_FUNC (panel_file_list_select_row),
 			    panel);
+	gtk_signal_connect (GTK_OBJECT (file_list), "unselect_row",
+			    GTK_SIGNAL_FUNC (panel_file_list_unselect_row),
+			    panel);
+#if 0
 	gtk_signal_connect (GTK_OBJECT (file_list), "button_press_event",
 			    GTK_SIGNAL_FUNC (panel_file_list_press_row),
 			    panel);
+#endif
 	gtk_clist_set_button_actions (GTK_CLIST (file_list), 1, GTK_BUTTON_SELECTS | GTK_BUTTON_DRAGS);
 	gtk_clist_set_button_actions (GTK_CLIST (file_list), 2, GTK_BUTTON_SELECTS);
 	
@@ -1180,15 +1262,16 @@ panel_create_file_list (WPanel *panel)
 
 	load_dnd_icons ();
 
-	gtk_drag_dest_set (GTK_WIDGET (file_list), GTK_DEST_DEFAULT_ALL,
+	gtk_drag_dest_set (GTK_WIDGET (file_list),
+			   GTK_DEST_DEFAULT_DROP,
 			   drop_types, ELEMENTS (drop_types),
-			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
-
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
+#if 0
 	/* Make directories draggable */
 	gtk_drag_source_set (GTK_WIDGET (file_list), GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
 			     drag_types, ELEMENTS (drag_types),
 			     GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_ASK);
-	
+#endif
 	gtk_signal_connect (GTK_OBJECT (file_list), "drag_data_get",
 			    GTK_SIGNAL_FUNC (panel_drag_data_get), panel);
 	gtk_signal_connect (GTK_OBJECT (file_list), "drag_data_delete",
@@ -1217,7 +1300,7 @@ panel_create_file_list (WPanel *panel)
 			    GTK_SIGNAL_FUNC (panel_clist_button_release), panel);
 
 	gtk_signal_connect (GTK_OBJECT (file_list), "motion_notify_event",
-			    GTK_SIGNAL_FUNC (panel_widget_motion), panel);
+			    GTK_SIGNAL_FUNC (panel_clist_motion), panel);
 	gtk_signal_connect (GTK_OBJECT (file_list), "drag_begin",
 			    GTK_SIGNAL_FUNC (panel_drag_begin), panel);
 	gtk_signal_connect (GTK_OBJECT (file_list), "drag_end",
@@ -1254,9 +1337,8 @@ panel_icon_list_select_icon (GtkWidget *widget, int index, GdkEvent *event, WPan
 			    panel->dir.list [index].f.link_to_dir){
 				fullname = concat_dir_and_file (panel->cwd, panel->dir.list [index].fname);
 				new_panel_at (fullname);
-				free (fullname);
+				g_free (fullname);
 			}
-			break;
 		} 
 		break;
 		
@@ -1286,8 +1368,8 @@ panel_icon_renamed (GtkWidget *widget, int index, char *dest, WPanel *panel)
 
 	source = panel->dir.list [index].fname;
 	if (mc_rename (source, dest) == 0){
-		free (panel->dir.list [index].fname);
-		panel->dir.list [index].fname = strdup (dest);
+		g_free (panel->dir.list [index].fname);
+		panel->dir.list [index].fname = g_strdup (dest);
 		return TRUE;
 	} else
 		return FALSE;
@@ -1346,14 +1428,13 @@ static GtkWidget *
 panel_create_icon_display (WPanel *panel)
 {
 	GnomeIconList *ilist;
-	GtkStyle *style;
 			       
-	ilist = GNOME_ICON_LIST (gnome_icon_list_new (90, NULL, TRUE));
-	/* Set the background of the icon list to white */
-	style = gtk_style_copy (gtk_widget_get_style (GTK_WIDGET (ilist)));
-	style->bg [GTK_STATE_NORMAL] = style->bg [GTK_STATE_PRELIGHT];
-	gtk_widget_set_style (GTK_WIDGET (ilist), style);
-		
+	ilist = GNOME_ICON_LIST (
+		gnome_icon_list_new_flags (
+			90, NULL,
+			GNOME_ICON_LIST_IS_EDITABLE|
+			GNOME_ICON_LIST_STATIC_TEXT));
+
 	gnome_icon_list_set_separators (ilist, " /-_.");
 	gnome_icon_list_set_row_spacing (ilist, 2);
 	gnome_icon_list_set_col_spacing (ilist, 2);
@@ -1378,9 +1459,10 @@ panel_create_icon_display (WPanel *panel)
 	load_imlib_icons ();
 	load_dnd_icons ();
 
-	gtk_drag_dest_set (GTK_WIDGET (ilist), GTK_DEST_DEFAULT_ALL,
+	gtk_drag_dest_set (GTK_WIDGET (ilist),
+			   GTK_DEST_DEFAULT_DROP,
 			   drop_types, ELEMENTS (drop_types),
-			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 
 	gtk_signal_connect (GTK_OBJECT (ilist), "drag_data_get",
 			    GTK_SIGNAL_FUNC (panel_drag_data_get),
@@ -1644,15 +1726,15 @@ display_mini_info (WPanel *panel)
 
 		link = concat_dir_and_file (panel->cwd, panel->dir.list [panel->selected].fname);
 		len = mc_readlink (link, link_target, MC_MAXPATHLEN);
-		free (link);
+		g_free (link);
 
 		if (len > 0){
 			link_target [len] = 0;
 			/* FIXME: Links should be handled differently */
-			/* str = copy_strings ("-> ", link_target, NULL); */
+			/* str = g_strconcat ("-> ", link_target, NULL); */
 			gnome_appbar_pop (bar);
 			gnome_appbar_push (bar, " ");
-			/*free (str); */
+			/* g_free (str); */
 		} else {
 			gnome_appbar_pop (bar);
 			gnome_appbar_push (bar, _("<readlink failed>"));
@@ -1664,12 +1746,12 @@ display_mini_info (WPanel *panel)
 		int  len = panel->estimated_total;
 		char *buffer;
 
-		buffer = xmalloc (len + 2, "display_mini_info");
+		buffer = g_malloc (len + 2);
 		format_file (buffer, panel, panel->selected, panel->estimated_total-2, 0, 1);
 		buffer [len] = 0;
 		gnome_appbar_pop (bar);
 		gnome_appbar_push (bar, buffer);
-		free (buffer);
+		g_free (buffer);
 	}
 	if (panel->list_type == list_icons){
 		if (panel->marked == 0){
@@ -1789,8 +1871,8 @@ panel_tree_scrolling_is_desirable (WPanel *panel, int x, int y)
 		if (va->value > va->lower)
 			return TRUE;
 	} else {
-		if (y > (GTK_WIDGET (dtree)->allocation.height-20)){
-			if (va->value < va->upper)
+		if (y > (GTK_WIDGET (dtree)->allocation.height - 10)){
+			if (va->value < va->upper - va->page_size)
 				return TRUE;
 		}
 	}
@@ -1877,14 +1959,24 @@ panel_tree_scroll (gpointer data)
 {
 	WPanel *panel = data;
 	GtkAdjustment *va;
+	double v;
 
 	va = scrolled_window_get_vadjustment (panel->tree_scrolled_window);
 
-	if (panel->drag_motion_y < 10)
-		gtk_adjustment_set_value (va, va->value - va->step_increment);
-	else{
-		gtk_adjustment_set_value (va, va->value + va->step_increment);
+	if (panel->drag_motion_y < 10) {
+		v = va->value - va->step_increment;
+		if (v < va->lower)
+			v = va->lower;
+
+		gtk_adjustment_set_value (va, v);
+	} else {
+		v = va->value + va->step_increment;
+		if (v > va->upper - va->page_size)
+			v = va->upper - va->page_size;
+
+		gtk_adjustment_set_value (va, v);
 	}
+
 	return TRUE;
 }
 
@@ -1901,7 +1993,7 @@ panel_tree_drag_motion (GtkWidget *widget, GdkDragContext *ctx, int x, int y, gu
 	WPanel *panel = data;
 	int r, row, col;
 
-	if (panel_setup_drag_motion (panel, x, y, panel_tree_scrolling_is_desirable, panel_tree_scroll))
+	if (panel_setup_drag_scroll (panel, x, y, panel_tree_scrolling_is_desirable, panel_tree_scroll))
 		return TRUE;
 
 	r = gtk_clist_get_selection_info (
@@ -1983,7 +2075,7 @@ panel_tree_drag_data_get (GtkWidget *widget, GdkDragContext *context,
 	switch (info){
 	case TARGET_URI_LIST:
 	case TARGET_TEXT_PLAIN:
-		data = copy_strings ("file:", dtree->drag_dir, NULL);
+		data = g_strconcat ("file:", dtree->drag_dir, NULL);
 		gtk_selection_data_set (
 			selection_data, selection_data->target, 8,
 			data, strlen (data)+1);
@@ -2012,7 +2104,7 @@ panel_create_tree_view (WPanel *panel)
 
 	gtk_drag_dest_set (GTK_WIDGET (tree), GTK_DEST_DEFAULT_ALL,
 			   drop_types, ELEMENTS (drop_types),
-			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 
 	/*
 	 * Drag and drop signals.
@@ -2029,9 +2121,9 @@ panel_create_tree_view (WPanel *panel)
 			    GTK_SIGNAL_FUNC (panel_tree_drag_data_get), panel);
 
 	/* Make directories draggable */
-	gtk_drag_source_set (GTK_WIDGET (tree), GDK_BUTTON1_MASK,
+	gtk_drag_source_set (GTK_WIDGET (tree), GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
 			     drag_types, ELEMENTS (drag_types),
-			     GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_ASK);
+			     GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_ASK | GDK_ACTION_DEFAULT);
 
 	/* Mouse is being moved over ourselves */
 	gtk_signal_connect (GTK_OBJECT (tree), "drag_motion",
@@ -2097,19 +2189,6 @@ panel_up (GtkWidget *button, WPanel *panel)
 	do_panel_cd (panel, "..", cd_exact);
 }
 
-static GtkWidget *
-button_switch_to (char **icon, GtkSignalFunc fn, void *closure)
-{
-	GtkWidget *button, *pix;
-
-	button = gtk_button_new ();
-	pix = gnome_pixmap_new_from_xpm_d (icon);
-	gtk_container_add (GTK_CONTAINER (button), pix);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked", fn, closure);
-
-	return button;
-}
-
 static void
 do_switch_to_iconic (GtkWidget *widget, WPanel *panel)
 {
@@ -2170,7 +2249,7 @@ static GnomeUIInfo viewbar[] = {
 		GNOME_APP_PIXMAP_DATA, listing_brief_list_xpm, 0, (GdkModifierType) 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Detailed"), N_("Switch view to show detailed file statistics"), do_switch_to_full_listing, NULL, NULL, \
 		GNOME_APP_PIXMAP_DATA, listing_list_xpm, 0, (GdkModifierType) 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Custom"), N_("Switch view to show custom determined statistics"), do_switch_to_custom_listing, NULL, NULL, \
+	{ GNOME_APP_UI_ITEM, N_("Custom"), N_("Switch view to show custom determined statistics."), do_switch_to_custom_listing, NULL, NULL, \
 		GNOME_APP_PIXMAP_DATA, listing_custom_xpm, 0, (GdkModifierType) 0, NULL },
 	GNOMEUIINFO_END
 };
@@ -2203,9 +2282,6 @@ do_ui_signal_connect (GnomeUIInfo *uiinfo, gchar *signal_name,
 static void
 tree_size_allocate (GtkWidget *widget, GtkAllocation *allocation, WPanel *panel)
 {
-	GtkWidget *tree = panel->tree;
-	GdkFont *tree_font = tree->style->font;
-	
 	if (allocation->width <= 0){
 		tree_panel_visible = 0;
 	} else {
@@ -2218,7 +2294,7 @@ void
 x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 {
 	GtkWidget *status_line, *filter, *vbox, *ministatus_box;
-	GtkWidget *cwd, *back_p, *fwd_p, *up_p;
+	GtkWidget *cwd;
 	GtkWidget *hbox, *evbox, *dock, *box;
 	GnomeUIBuilderData uibdata;
 
@@ -2316,6 +2392,13 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	panel->up_b     = toolbar[1].widget;
 	panel->fwd_b    = toolbar[2].widget;
 	panel_update_marks (panel);
+	if (panel->list_type == list_brief)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (viewbar[1].widget), TRUE);
+	else if (panel->list_type == list_full)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (viewbar[2].widget), TRUE);
+	else if (panel->list_type == list_user)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (viewbar[3].widget), TRUE);
+
 	/*
 	 * Here we make the view buttons.
 	 */
@@ -2328,7 +2411,7 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 				   evbox,
 				   NULL, NULL);
 	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("Location:")), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), cwd, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), cwd, TRUE, TRUE, 0);
 	dock =  gnome_dock_item_new ("gmc-toolbar1", GNOME_DOCK_ITEM_BEH_EXCLUSIVE | GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL);
 	gtk_container_add (GTK_CONTAINER(dock),status_line);
 	gnome_dock_add_item (GNOME_DOCK(GNOME_APP (panel->xwindow)->dock),
@@ -2470,14 +2553,14 @@ panel_update_cols (Widget *panel, int frame_size)
 char *
 get_nth_panel_name (int num)
 {
-    static char buffer [20];
+    static char buffer [BUF_TINY];
 
     if (!num)
         return "New Left Panel";
     else if (num == 1)
         return "New Right Panel";
     else {
-        sprintf (buffer, "%ith Panel", num);
+        g_snprintf (buffer, sizeof (buffer), "%ith Panel", num);
         return buffer;
     }
 }
@@ -2490,7 +2573,7 @@ load_hint (void)
 	if ((hint = get_random_hint ())){
 		if (*hint)
 			set_hintbar (hint);
-		free (hint);
+		g_free (hint);
 	} else
 		set_hintbar ("The GNOME File Manager " VERSION);
 
