@@ -20,22 +20,15 @@
 
 #include <config.h>
 #include "tty.h"
-#include "fs.h"
 #include <sys/param.h>
 #include <string.h>
-#include <stdlib.h>	/* For malloc() and free() */
 #include <stdio.h>
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>	/* For chdir(), readlink() and getwd()/getcwd() */
 #endif
-/*#include "mc.h"*/
-
-#include "mem.h"
-#include "mad.h"
 #include "global.h"
 #include "dir.h"
-#include "util.h"
 #include "panel.h"
 #include "color.h"
 #include "tree.h"
@@ -54,7 +47,6 @@
 #include "widget.h"
 #include "../vfs/vfs.h"
 #include "../vfs/extfs.h"
-#include <glib.h>
 #ifdef NEEDS_DRIVE_H
 # include "drive.h"
 #endif
@@ -151,7 +143,7 @@ delete_format (format_e *format)
     
     while (format){
         next = format->next;
-        free (format);
+        g_free (format);
         format = next;
      }
 }
@@ -218,7 +210,59 @@ file_entry_color (file_entry *fe)
     return (NORMAL_COLOR);
 }
 
+/* String representations of various file attributes */
+/* name */
+char *
+string_file_name (file_entry *fe, int len)
+{
+	return fe->fname;
+}
+
+/* size */
+char *
+string_file_size (file_entry *fe, int len)
+{
+    static char buffer [BUF_TINY];
+    int i;
+
+#ifdef HAVE_ST_RDEV
+    if (S_ISBLK (fe->buf.st_mode) || S_ISCHR (fe->buf.st_mode))
+        g_snprintf (buffer, sizeof (buffer), "%3d,%3d", 
+		    (int) ((fe->buf.st_rdev >> 8) & 0xff), 
+		    (int) (fe->buf.st_rdev & 0xff));
+    else
+#endif 
+    {   
+        g_snprintf (buffer, sizeof (buffer), "%lu", (unsigned long) fe->buf.st_size);
+        if (len && (i = strlen (buffer)) > len) {
+            if (i - 2 > len) {
+                if (i - 5 > len)
+                    g_snprintf (buffer, sizeof (buffer), "%luG", (unsigned long) ((fe->buf.st_size) >> 30));
+                else
+                    g_snprintf (buffer, sizeof (buffer), "%luM", (unsigned long) ((fe->buf.st_size) >> 20));
+            } else
+                g_snprintf (buffer, sizeof (buffer), "%luK", (unsigned long) ((fe->buf.st_size) >> 10));
+        }
+    }
+    return buffer;
+}
+
+/* bsize */
+char *
+string_file_size_brief (file_entry *fe, int len)
+{
+    static char buffer [BUF_TINY];
+
+    if (S_ISDIR (fe->buf.st_mode)){
+       strcpy (buffer, (strcmp (fe->fname, "..") ? "SUB-DIR" : "UP--DIR"));
+       return buffer;
+    }
+
+    return string_file_size (fe, len);
+}
+
 /* This functions return a string representation of a file entry */
+/* type */
 char *
 string_file_type (file_entry *fe, int len)
 {
@@ -249,179 +293,117 @@ string_file_type (file_entry *fe, int len)
     return buffer;
 }
 
-char *
-string_file_size_brief (file_entry *fe, int len)
-{
-    static char buffer [8];
-
-    if (S_ISDIR (fe->buf.st_mode)){
-       strcpy (buffer, (strcmp (fe->fname, "..") ? "SUB-DIR" : "UP--DIR"));
-       return buffer;
-    }
-
-    return string_file_size (fe, len);
-}
-
-char *
-string_file_permission (file_entry *fe, int len)
-{
-    return string_perm (fe->buf.st_mode);
-}
-
-char *
-string_file_nlinks (file_entry *fe, int len)
-{
-    static char buffer [20];
-
-    sprintf (buffer, "%16d", fe->buf.st_nlink);
-    return buffer;
-}
-
-char *
-string_file_size (file_entry *fe, int len)
-{
-    static char buffer [16];
-    int i;
-
-#ifdef HAVE_ST_RDEV
-    if (S_ISBLK (fe->buf.st_mode) || S_ISCHR (fe->buf.st_mode))
-        sprintf (buffer, "%3d,%3d", (int) ((fe->buf.st_rdev >> 8) & 0xff), 
-            (int) (fe->buf.st_rdev & 0xff));
-    else
-#endif 
-    {   
-        sprintf (buffer, "%lu", (unsigned long) fe->buf.st_size);
-        if (len && (i = strlen (buffer)) > len) {
-            if (i - 2 > len) {
-                if (i - 5 > len)
-                    sprintf (buffer, "%luG", (unsigned long) ((fe->buf.st_size) >> 30));
-                else
-                    sprintf (buffer, "%luM", (unsigned long) ((fe->buf.st_size) >> 20));
-            } else
-                sprintf (buffer, "%luK", (unsigned long) ((fe->buf.st_size) >> 10));
-        }
-    }
-    return buffer;
-}
-
+/* mtime */
 char *
 string_file_mtime (file_entry *fe, int len)
 {
     return file_date (fe->buf.st_mtime);
 }
 
+/* atime */
 char *
 string_file_atime (file_entry *fe, int len)
 {
     return file_date (fe->buf.st_atime);
 }
 
+/* ctime */
 char *
 string_file_ctime (file_entry *fe, int len)
 {
     return file_date (fe->buf.st_ctime);
 }
 
-#ifdef HAVE_GNOME
-/* In GNOME, the CList truncates the names */
+/* perm */
+char *
+string_file_permission (file_entry *fe, int len)
+{
+    return string_perm (fe->buf.st_mode);
+}
+
+/* mode */
+char *
+string_file_perm_octal (file_entry *fe, int len)
+{
+    static char buffer [10];
+
+    g_snprintf (buffer, sizeof (buffer), "0%06o", fe->buf.st_mode);
+    return buffer;
+}
+
+/* nlink */
+char *
+string_file_nlinks (file_entry *fe, int len)
+{
+    static char buffer [BUF_TINY];
+
+    g_snprintf (buffer, sizeof (buffer), "%16d", fe->buf.st_nlink);
+    return buffer;
+}
+
+/* inode */
+char *
+string_inode (file_entry *fe, int len)
+{
+    static char buffer [10];
+
+    g_snprintf (buffer, sizeof (buffer), "%lu", (unsigned long) fe->buf.st_ino);
+    return buffer;
+}
+
+/* nuid */
+char *
+string_file_nuid (file_entry *fe, int len)
+{
+    static char buffer [10];
+
+    g_snprintf (buffer, sizeof (buffer), "%d", fe->buf.st_uid);
+    return buffer;
+}
+
+/* ngid */
+char *
+string_file_ngid (file_entry *fe, int len)
+{
+    static char buffer [10];
+
+    g_snprintf (buffer, sizeof (buffer), "%d", fe->buf.st_gid);
+    return buffer;
+}
+
+/* owner */
 char *
 string_file_owner (file_entry *fe, int len)
 {
     return get_owner (fe->buf.st_uid);
 }
 
+/* group */
 char *
 string_file_group (file_entry *fe, int len)
 {
     return get_group (fe->buf.st_gid);
 }
 
-char *
-string_file_name (file_entry *fe, int len)
-{
-	return fe->fname;
-}
-#else
-char *
-string_file_owner (file_entry *fe, int len)
-{
-    if (len)
-	return name_trunc (get_owner (fe->buf.st_uid), len);
-    else
-	return get_owner (fe->buf.st_uid);
-}
-
-char *
-string_file_group (file_entry *fe, int len)
-{
-    if (len)
-	return name_trunc (get_group (fe->buf.st_gid), len);
-    else
-	return get_group (fe->buf.st_gid);
-}
-
-char *
-string_file_name (file_entry *fe, int len)
-{
-    if (len)
-        return name_trunc (fe->fname, len);
-    else
-	return fe->fname;
-}
-#endif
-
-char *
-string_space (file_entry *fe, int len)
-{
-    return " ";
-}
-
-char *
-string_dot (file_entry *fe, int len)
-{
-    return ".";
-}
-
+/* mark */
 char *
 string_marked (file_entry *fe, int len)
 {
     return fe->f.marked ? "*" : " ";
 }
 
+/* space */
 char *
-string_file_perm_octal (file_entry *fe, int len)
+string_space (file_entry *fe, int len)
 {
-    static char buffer [9];
-
-    sprintf (buffer, "0%06o", fe->buf.st_mode);
-    return buffer;
+    return " ";
 }
 
+/* dot */
 char *
-string_inode (file_entry *fe, int len)
+string_dot (file_entry *fe, int len)
 {
-    static char buffer [9];
-
-    sprintf (buffer, "%ld", (long) fe->buf.st_ino);
-    return buffer;
-}
-
-char *
-string_file_ngid (file_entry *fe, int len)
-{
-    static char buffer [9];
-
-    sprintf (buffer, "%d", fe->buf.st_gid);
-    return buffer;
-}
-
-char *
-string_file_nuid (file_entry *fe, int len)
-{
-    static char buffer [9];
-
-    sprintf (buffer, "%d", fe->buf.st_uid);
-    return buffer;
+    return ".";
 }
 
 #ifdef HAVE_GNOME
@@ -440,52 +422,60 @@ static struct {
     char *(*string_fn)(file_entry *, int);
     sortfn *sort_routine;
 } formats [] = {
-{ "name",  12, 1, J_LEFT,  N_("Name"),       1, string_file_name,       (sortfn *) sort_name },
-{ "size",  7,  0, J_RIGHT, N_("Size"),       1, string_file_size,       (sortfn *) sort_size },
-{ "type",  GT, 0, J_LEFT,     "",            1, string_file_type,       (sortfn *) sort_type },
-{ "mtime", 12, 0, J_RIGHT, N_("MTime"),      1, string_file_mtime,      (sortfn *) sort_time },
-{ "bsize", 7,  0, J_RIGHT, N_("Size"),       1, string_file_size_brief, (sortfn *) sort_size },
-{ "perm",  10, 0, J_LEFT,  N_("Permission"), 1, string_file_permission, NULL },
-{ "mode",  6,  0, J_RIGHT, N_("Perm"),       1, string_file_perm_octal, NULL },
-{ "|",     1,  0, J_RIGHT, N_("|"),          0, 0,                      NULL },
-{ "nlink", 2,  0, J_RIGHT, N_("Nl"),         1, string_file_nlinks,     (sortfn *) sort_links },
-{ "ngid",  5,  0, J_RIGHT, N_("GID"),        1, string_file_ngid,       (sortfn *) sort_ngid },
-{ "nuid",  5,  0, J_RIGHT, N_("UID"),        1, string_file_nuid,       (sortfn *) sort_nuid },
-{ "owner", 8,  0, J_LEFT,  N_("Owner"),      1, string_file_owner,      (sortfn *) sort_owner },
-{ "group", 8,  0, J_LEFT,  N_("Group"),      1, string_file_group,      (sortfn *) sort_group },
-{ "atime", 12, 0, J_RIGHT, N_("ATime"),      1, string_file_atime,      (sortfn *) sort_atime },
-{ "ctime", 12, 0, J_RIGHT, N_("CTime"),      1, string_file_ctime,      (sortfn *) sort_ctime },
-{ "space", 1,  0, J_RIGHT,    " ",          0, string_space,           NULL },
-{ "dot",   1,  0, J_RIGHT,    " ",          0, string_dot,             NULL },
-{ "mark",  1,  0, J_RIGHT,    " ",          1, string_marked,          NULL },
-{ "inode", 5,  0, J_RIGHT, N_("Inode"),      1, string_inode,           (sortfn *) sort_inode },
+{ "name",  12, 1, J_LEFT_FIT,	N_("Name"),	1, string_file_name,	   (sortfn *) sort_name },
+{ "size",  7,  0, J_RIGHT,	N_("Size"),	1, string_file_size,	   (sortfn *) sort_size },
+{ "bsize", 7,  0, J_RIGHT,	N_("Size"),	1, string_file_size_brief, (sortfn *) sort_size },
+{ "type",  GT, 0, J_LEFT,	"",		1, string_file_type,	   (sortfn *) sort_type },
+{ "mtime", 12, 0, J_RIGHT,	N_("MTime"),	1, string_file_mtime,	   (sortfn *) sort_time },
+{ "atime", 12, 0, J_RIGHT,	N_("ATime"),	1, string_file_atime,	   (sortfn *) sort_atime },
+{ "ctime", 12, 0, J_RIGHT,	N_("CTime"),	1, string_file_ctime,	   (sortfn *) sort_ctime },
+{ "perm",  10, 0, J_LEFT,	N_("Permission"),1,string_file_permission, NULL },
+{ "mode",  6,  0, J_RIGHT,	N_("Perm"),	1, string_file_perm_octal, NULL },
+{ "nlink", 2,  0, J_RIGHT,	N_("Nl"),	1, string_file_nlinks,	   (sortfn *) sort_links },
+{ "inode", 5,  0, J_RIGHT,	N_("Inode"),	1, string_inode,	   (sortfn *) sort_inode },
+{ "nuid",  5,  0, J_RIGHT,	N_("UID"),	1, string_file_nuid,	   (sortfn *) sort_nuid },
+{ "ngid",  5,  0, J_RIGHT,	N_("GID"),	1, string_file_ngid,	   (sortfn *) sort_ngid },
+{ "owner", 8,  0, J_LEFT_FIT,	N_("Owner"),	1, string_file_owner,	   (sortfn *) sort_owner },
+{ "group", 8,  0, J_LEFT_FIT,	N_("Group"),	1, string_file_group,	   (sortfn *) sort_group },
+{ "mark",  1,  0, J_RIGHT,	" ",		1, string_marked,	   NULL },
+{ "|",     1,  0, J_RIGHT,	" ",		0, NULL,		   NULL },
+{ "space", 1,  0, J_RIGHT,	" ",		0, string_space,	   NULL },
+{ "dot",   1,  0, J_RIGHT,	" ",		0, string_dot,		   NULL },
 };
 
 static char *
 to_buffer (char *dest, int just_mode, int len, char *txt)
 {
     int txtlen = strlen (txt);
-    int still;
+    int still, over;
+    
+    memset (dest, ' ', len);
 
-    if (txtlen > len){
-	if (just_mode != J_LEFT)
-	    txt += txtlen - len;
-	txtlen = len;
-    }
-    still = len - txtlen;
-    if (just_mode == J_LEFT){
-	strcpy (dest, txt);
-	dest += txtlen;
-	while (still--)
-	    *dest++ = ' ';
-	*dest = 0;
-    } else {
-	while (still--)
-	    *dest++ = ' ';
-	strcpy (dest, txt);
-	dest += txtlen;
-    }
-    return dest;
+    still = (over=(txtlen > len)) ? (txtlen - len) : (len - txtlen);
+    
+    switch (HIDE_FIT(just_mode)){
+        case J_LEFT:
+	    still = 0;
+	    break;
+	case J_CENTER:
+	    still /= 2;
+	    break;
+	case J_RIGHT:
+	default:
+	    break;
+    }    
+
+    if (over){
+	if (IS_FIT(just_mode))
+	    strcpy (dest, name_trunc(txt, len));
+	else
+	    strncpy (dest, txt+still, len);
+    } else 
+	strncpy (dest+still, txt, txtlen);
+    
+    dest[len] = '\0';
+    
+    return (dest + len);
 }
 
 int
@@ -554,10 +544,6 @@ format_file (char *dest, WPanel *panel, int file_index, int width, int attr, int
 	    if (length == width)
 		    break;
 #else
-            /* What shall we do? Will we color each line according to
-	     * the file type? Any suggestions to mc@timur.kazan.su
-	     */
-	    
             attrset (color);
  
             if (permission_mode && !strcmp(format->id, "perm"))
@@ -569,7 +555,6 @@ format_file (char *dest, WPanel *panel, int file_index, int width, int attr, int
 #endif
 	} else {
 #ifndef HAVE_X
-            /* I'm preffer the view without this 3 lines, try to kill it :-) */
             if (attr == SELECTED || attr == MARKED_SELECTED)
                 attrset (SELECTED_COLOR);
             else
@@ -600,7 +585,7 @@ repaint_file (WPanel *panel, int file_index, int mv, int attr, int isstatus)
 {
     int    second_column = 0;
     int	   width, offset;
-    char   buffer [255];
+    char   buffer [BUF_MEDIUM];
 
     offset = 0;
     if (!isstatus && panel->split){
@@ -656,13 +641,13 @@ display_mini_info (WPanel *panel)
 
     /* Status displays total marked size */
     if (panel->marked){
-	char buffer [100];
+	char buffer [BUF_SMALL];
 	char *p;
 	
 	attrset (MARKED_COLOR);
 	printw  ("%*s", panel->widget.cols-2, " ");
 	widget_move (&panel->widget, llines (panel)+3, 1);
-	sprintf (buffer, _("  %s bytes in %d file%s"),
+	g_snprintf (buffer, sizeof (buffer), _("  %s bytes in %d file%s"),
 		 size_trunc_sep (panel->total), panel->marked,
 		 panel->marked == 1 ? "" : "s");
 	p = buffer;
@@ -683,7 +668,7 @@ display_mini_info (WPanel *panel)
 
 	link = concat_dir_and_file (panel->cwd, panel->dir.list [panel->selected].fname);
 	len = mc_readlink (link, link_target, MC_MAXPATHLEN);
-	free (link);
+	g_free (link);
 	if (len > 0){
 	    link_target[len] = 0;
 	    printw ("-> %-*s", panel->widget.cols - 5,
@@ -867,7 +852,7 @@ Xtry_to_select (WPanel *panel, char *name)
     for (i = 0; i < panel->count; i++){
 	if (strcmp (subdir, panel->dir.list [i].fname) == 0) {
 	    do_select (panel, i);
-            free (subdir);
+            g_free (subdir);
 	    return;
         }
     }
@@ -875,7 +860,7 @@ Xtry_to_select (WPanel *panel, char *name)
     /* Try to select a file near the file that is missing */
     if (panel->selected >= panel->count)
         do_select (panel, panel->count-1);
-    free (subdir);
+    g_free (subdir);
 }
 
 #ifndef PORT_HAS_PANEL_UPDATE_COLS
@@ -914,9 +899,9 @@ panel_save_name (WPanel *panel)
     
     /* If the program is shuting down */
     if ((midnight_shutdown && auto_save_setup) || saving_setup)
-	return copy_strings (panel->panel_name, 0);
+	return  g_strconcat (panel->panel_name, NULL);
     else
-	return copy_strings ("Temporal:", panel->panel_name, 0);
+	return  g_strconcat ("Temporal:", panel->panel_name, NULL);
 }
 
 static void
@@ -940,21 +925,21 @@ panel_destroy (WPanel *p)
 	while (current){
 	    old = current;
 	    current = current->prev;
-	    free (old->text);
-	    free (old);
+	    g_free (old->text);
+	    g_free (old);
 	}
     }
-    free (p->hist_name);
+    g_free (p->hist_name);
 
     delete_format (p->format);
     delete_format (p->status_format);
     
-    free (p->user_format);
+    g_free (p->user_format);
     for (i = 0; i < LIST_TYPES; i++)
-	free (p->user_status_format [i]);
-    free (p->dir.list);
-    free (p->panel_name);
-    free (name);
+	g_free (p->user_status_format [i]);
+    g_free (p->dir.list);
+    g_free (p->panel_name);
+    g_free (name);
 }
 
 static void
@@ -981,8 +966,7 @@ panel_new (char *panel_name)
     char *section;
     int i, err;
 
-    panel = xmalloc (sizeof (WPanel), "panel_new");
-    memset (panel, 0, sizeof (WPanel));
+    panel = g_new0 (WPanel, 1);
     
     /* No know sizes of the panel at startup */
     init_widget (&panel->widget, 0, 0, 0, 0, (callback_fn)
@@ -995,11 +979,11 @@ panel_new (char *panel_name)
     mc_get_current_wd (panel->cwd, sizeof (panel->cwd)-2);
     strcpy (panel->lwd, ".");
 
-    panel->hist_name = copy_strings ("Dir Hist ", panel_name, 0);
+    panel->hist_name = g_strconcat ("Dir Hist ", panel_name, NULL);
     panel->dir_history = history_get (panel->hist_name);
     directory_history_add (panel, panel->cwd);
 
-    panel->dir.list         = (file_entry *) malloc (MIN_FILES * sizeof (file_entry));
+    panel->dir.list         = g_new (file_entry, MIN_FILES);
     panel->dir.size         = MIN_FILES;
     panel->active           = 0;
     panel->filter           = 0;
@@ -1017,32 +1001,32 @@ panel_new (char *panel_name)
     panel->status_format    = 0;    
     panel->format_modified  = 1;
     
-    panel->panel_name = strdup (panel_name);
-    panel->user_format = strdup (DEFAULT_USER_FORMAT);
+    panel->panel_name = g_strdup (panel_name);
+    panel->user_format = g_strdup (DEFAULT_USER_FORMAT);
     
     for(i = 0; i < LIST_TYPES; i++)
-	panel->user_status_format [i] = strdup (DEFAULT_USER_FORMAT);
+	panel->user_status_format [i] = g_strdup (DEFAULT_USER_FORMAT);
     
     panel->search_buffer [0] = 0;
     panel->frame_size = frame_half;
-    section = copy_strings ("Temporal:", panel->panel_name, 0);
+    section = g_strconcat ("Temporal:", panel->panel_name, NULL);
     if (!profile_has_section (section, profile_name)){
-	free (section);
-	section = strdup (panel->panel_name);
+	g_free (section);
+	section = g_strdup (panel->panel_name);
     }
     panel_load_setup (panel, section);
-    free (section);
+    g_free (section);
 
     /* Load format strings */
     err = set_panel_formats (panel);
     if (err){
 	if (err & 0x01){
-	    free (panel->user_format);
-	    panel->user_format = strdup (DEFAULT_USER_FORMAT);
+	    g_free (panel->user_format);
+	    panel->user_format = g_strdup (DEFAULT_USER_FORMAT);
 	}
 	if (err & 0x02){
-	    free (panel->user_status_format [panel->list_type]);
-	    panel->user_status_format [panel->list_type] = strdup (DEFAULT_USER_FORMAT);
+	    g_free (panel->user_status_format [panel->list_type]);
+	    panel->user_status_format [panel->list_type] = g_strdup (DEFAULT_USER_FORMAT);
 	}
 	set_panel_formats (panel);
     }
@@ -1077,7 +1061,7 @@ panel_reload (WPanel *panel)
 	    strcpy (panel->cwd, PATH_SEP_STR);
 	else
 	    *last_slash = 0;
-        bzero (&(panel->dir_stat), sizeof (panel->dir_stat));
+        memset (&(panel->dir_stat), 0, sizeof (panel->dir_stat));
 	show_dir (panel);
     }
     
@@ -1222,7 +1206,7 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus, i
     while (*format){           /* format can be an empty string */
 	int found = 0;
 
-        darr = xmalloc (sizeof (format_e), "parse_display_format");
+        darr = g_new (format_e, 1);
 	
         /* I'm so ugly, don't look at me :-) */
         if (!home)
@@ -1302,7 +1286,7 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus, i
 	    delete_format (home);
 	    old_char = format [pos];
 	    format [pos] = 0;
-	    *error = copy_strings(_("Unknow tag on display format: "), format, 0);
+	    *error = g_strconcat (_("Unknow tag on display format: "), format, NULL);
 	    format [pos] = old_char;
 	    return 0;
 	}
@@ -1398,7 +1382,7 @@ set_panel_formats (WPanel *p)
     form = use_display_format (p, panel_format (p), &err, 0);
 	
     if (err){
-        free (err);
+        g_free (err);
         retcode = 1;
     }
     else {
@@ -1413,7 +1397,7 @@ set_panel_formats (WPanel *p)
 	form = use_display_format (p, mini_status_format (p), &err, 1);
 	
 	if (err){
-	    free (err);
+	    g_free (err);
 	    retcode += 2;
 	}
 	else {
@@ -2060,13 +2044,13 @@ do_enter_on_file_entry (file_entry *fe)
 #endif	    
 	    {
 	        char *tmp = name_quote (fe->fname, 0);
-	        char *cmd = copy_strings (".", PATH_SEP_STR, tmp, 0);
+	        char *cmd = g_strconcat (".", PATH_SEP_STR, tmp, NULL);
 	        if (!confirm_execute || (query_dialog (_(" The Midnight Commander "),
 						       _(" Do you really want to execute? "),
 						       0, 2, _("&Yes"), _("&No")) == 0))
 	            execute (cmd);
-	        free (tmp);
-	        free (cmd);
+	        g_free (tmp);
+	        g_free (cmd);
 	    } 
 #ifdef USE_VFS	    	    
 	    else {
@@ -2079,7 +2063,7 @@ do_enter_on_file_entry (file_entry *fe)
 		if (!mc_setctl (tmp, MCCTL_EXTFS_RUN, NULL))
 		    message (1, _(" Warning "), _(" No action taken "));
 
-	        free (tmp);
+	        g_free (tmp);
 	    }
 #endif /* USE_VFS */	    
 	    return 1;
@@ -2127,7 +2111,7 @@ chdir_other_panel (WPanel *panel)
 
     move_down (panel);
     
-    free (new_dir);
+    g_free (new_dir);
 }
 
 static void
@@ -2160,7 +2144,7 @@ chdir_to_readlink (WPanel *panel)
 	    p[1] = 0;
 	}
 	if (*buffer == PATH_SEP)
-	    new_dir = strdup (buffer);
+	    new_dir = g_strdup (buffer);
 	else
 	    new_dir = concat_dir_and_file (panel->cwd, buffer);
 
@@ -2170,7 +2154,7 @@ chdir_to_readlink (WPanel *panel)
 	
 	move_down (panel);
 	    
-	free (new_dir);
+	g_free (new_dir);
     }
 }
 
@@ -2567,7 +2551,7 @@ panel_re_sort (WPanel *panel)
     if (panel == NULL)
 	    return;
     
-    filename = strdup (selection (panel)->fname);
+    filename = g_strdup (selection (panel)->fname);
     unselect_item (panel);
     do_sort (&panel->dir, panel->sort_type, panel->count-1, panel->reverse, panel->case_sensitive);
     panel->selected = -1;
@@ -2577,7 +2561,7 @@ panel_re_sort (WPanel *panel)
 	    break;
 	}
     }
-    free (filename);
+    g_free (filename);
     panel->top_file = panel->selected - ITEMS (panel)/2;
     if (panel->top_file < 0)
 	panel->top_file = 0;
@@ -2597,10 +2581,10 @@ panel_set_sort_order (WPanel *panel, sortfn *sort_order)
     if (sort_order == (sortfn *) unsorted){
 	char *current_file;
 	
-	current_file = strdup (panel->dir.list [panel->selected].fname);
+	current_file = g_strdup (panel->dir.list [panel->selected].fname);
 	panel_reload (panel);
 	try_to_select (panel, current_file);
-	free (current_file);
+	g_free (current_file);
     }
     panel_re_sort (panel);
 }
