@@ -41,13 +41,10 @@
 #include <errno.h>
 #include <dirent.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <glib.h>
-#include "fs.h"
-#include "../vfs/vfs.h"
-#include "util.h"
+#include "global.h"
 #include "treestore.h"
+#include "../vfs/vfs.h"
 #ifdef OS2_NT
 #   include <io.h>
 #endif
@@ -154,8 +151,8 @@ tree_store_destroy (void)
 	while (current){
 		old = current;
 		current = current->next;
-		free (old->name);
-		free (old);
+		g_free (old->name);
+		g_free (old);
 	}
 
 	ts.tree_first = NULL;
@@ -166,7 +163,7 @@ tree_store_destroy (void)
 static char *
 decode (char *buffer)
 {
-	char *res = strdup (buffer);
+	char *res = g_strdup (buffer);
 	char *p, *q;
 	
 	for (p = q = res; *p; p++, q++){
@@ -274,7 +271,7 @@ tree_store_load (char *name)
 					e->scanned = scanned;
 					strcpy (oldname, name);
 				}
-			free (name);
+			g_free (name);
 		}
 		fclose (file);
 	}
@@ -301,7 +298,7 @@ encode (char *string)
 			special_chars++;
 	}
 	
-	res = malloc (p - string + special_chars + 1);
+	res = g_malloc (p - string + special_chars + 1);
 	for (p = string, q = res; *p; p++, q++){
 		if (*p != '\n' && *p != '\\'){
 			*q = *p;
@@ -345,12 +342,12 @@ tree_store_save (char *name)
 			char *encoded = encode (current->name + common);
 			
 			i = fprintf (file, "%d:%d %s\n", current->scanned, common, encoded);
-			free (encoded);
+			g_free (encoded);
 		} else {
 			char *encoded = encode (current->name);
 			
 			i = fprintf (file, "%d:%s\n", current->scanned, encoded);
-			free (encoded);
+			g_free (encoded);
 		}
 		
 		if (i == EOF){
@@ -389,7 +386,7 @@ tree_store_add_entry (char *name)
 		return current; /* Already in the list */
 
 	/* Not in the list -> add it */
-	new = xmalloc (sizeof (tree_entry), "ts, tree_entry");
+	new = g_new (tree_entry, 1);
 	if (!current){
 		/* Append to the end of the list */
 		if (!ts.tree_first){
@@ -418,7 +415,7 @@ tree_store_add_entry (char *name)
 	}
 
 	/* Calculate attributes */
-	new->name = strdup (name);
+	new->name = g_strdup (name);
 	len = strlen (new->name);
 	new->sublevel = 0;
 	for (i = 0; i < len; i++)
@@ -446,7 +443,7 @@ tree_store_add_entry (char *name)
 	
 	if (new->sublevel > 1){
 		/* Let's check if the parent directory is in the tree */
-		char *parent = strdup (new->name);
+		char *parent = g_strdup (new->name);
 		int i;
 		
 		for (i = strlen (parent) - 1; i > 1; i--){
@@ -456,7 +453,7 @@ tree_store_add_entry (char *name)
 				break;
 			}
 		}
-		free (parent);
+		g_free (parent);
 	}
 
 	tree_store_dirty (TRUE);
@@ -493,8 +490,8 @@ remove_entry (tree_entry *entry)
 		ts.tree_last = entry->prev;
 	
 	/* Free the memory used by the entry */
-	free (entry->name);
-	free (entry);
+	g_free (entry->name);
+	g_free (entry);
 
 	return ret;
 }
@@ -544,13 +541,14 @@ tree_store_mark_checked (const char *subname)
 	
 	if (!ts.loaded)
 		return;
+	g_return_if_fail (ts.check_name != NULL);
 
 	/* Calculate the full name of the subdirectory */
 	if (subname [0] == '.' &&
 	    (subname [1] == 0 || (subname [1] == '.' && subname [2] == 0)))
 		return;
 	if (ts.check_name [0] == PATH_SEP && ts.check_name [1] == 0)
-		name = copy_strings (PATH_SEP_STR, subname, 0);
+		name = g_strconcat (PATH_SEP_STR, subname, NULL);
 	else
 		name = concat_dir_and_file (ts.check_name, subname);
 	
@@ -559,10 +557,12 @@ tree_store_mark_checked (const char *subname)
 	while (current && (flag = pathcmp (current->name, name)) < 0)
 		current = current->next;
 	
-	if (flag != 0)
+	if (flag != 0){
 		/* Doesn't exist -> add it */
 		current = tree_store_add_entry (name);
-	free (name);
+		tree_store_notify_add (current);
+	}
+	g_free (name);
 	
 	/* Clear the deletion mark from the subdirectory and its children */
 	base = current;
@@ -596,19 +596,19 @@ tree_store_start_check (char *path)
 	if (!current){
 		struct stat s;
 
-		if (stat (path, &s) == -1)
+		if (mc_stat (path, &s) == -1)
 			return NULL;
 
 		if (!S_ISDIR (s.st_mode))
 			return NULL;
 		
 		current = tree_store_add_entry (path);
-		ts.check_name = strdup (path);
+		ts.check_name = g_strdup (path);
 
 		return current;
 	}
 
-	ts.check_name = strdup (path);
+	ts.check_name = g_strdup (path);
 	
 	retval = current;
 	
@@ -645,6 +645,8 @@ tree_store_end_check (void)
 
 	if (!ts.loaded)
 		return;
+
+	g_return_if_fail (ts.check_name);
 	
 	/* Check delete marks and delete if found */
 	len = strlen (ts.check_name);
@@ -659,7 +661,7 @@ tree_store_end_check (void)
 			remove_entry (old);
 	}
 
-	free (ts.check_name);
+	g_free (ts.check_name);
 	ts.check_name = NULL;
 }
 
@@ -686,11 +688,11 @@ tree_store_rescan (char *dir)
 				continue;
 
 			full_name = concat_dir_and_file (dir, dp->d_name);
-			if (lstat (full_name, &buf) != -1){
+			if (mc_lstat (full_name, &buf) != -1){
 				if (S_ISDIR (buf.st_mode))
 					tree_store_mark_checked (dp->d_name);
 			}
-			free (full_name);
+			g_free (full_name);
 		}
 		mc_closedir (dirp);
 	}
@@ -701,6 +703,7 @@ tree_store_rescan (char *dir)
 }      
 
 static Hook *remove_entry_hooks;
+static Hook *add_entry_hooks;
 
 void
 tree_store_add_entry_remove_hook (tree_store_remove_fn callback, void *data)
@@ -708,6 +711,12 @@ tree_store_add_entry_remove_hook (tree_store_remove_fn callback, void *data)
 	add_hook (&remove_entry_hooks, (void (*)(void *))callback, data);
 }
 
+void
+tree_store_remove_entry_remove_hook (tree_store_remove_fn callback)
+{
+	delete_hook (&remove_entry_hooks, (void (*)(void *))callback);
+}
+
 void
 tree_store_notify_remove (tree_entry *entry)
 {
@@ -722,6 +731,32 @@ tree_store_notify_remove (tree_entry *entry)
 	}
 }
 
+void
+tree_store_add_entry_add_hook (tree_store_add_fn callback, void *data)
+{
+	add_hook (&add_entry_hooks, (void (*)(void *))callback, data);
+}
+
+void
+tree_store_remove_entry_add_hook (tree_store_add_fn callback)
+{
+	delete_hook (&add_entry_hooks, (void (*)(void *))callback);
+}
+
+void
+tree_store_notify_add (tree_entry *entry)
+{
+	Hook *p = add_entry_hooks;
+	tree_store_add_fn r;
+
+	
+	while (p){
+		r = (tree_store_add_fn) p->hook_fn;
+		r (entry, p->hook_data);
+		p = p->next;
+	}
+}
+
 tree_scan *
 tree_store_opendir (char *path)
 {
@@ -739,7 +774,7 @@ tree_store_opendir (char *path)
 	if (entry->next == NULL)
 		return NULL;
 	
-	scan = xmalloc (sizeof (tree_scan), "");
+	scan = g_new (tree_scan, 1);
 	scan->base = entry;
 	scan->current = entry->next;
 	scan->sublevel = entry->next->sublevel;
@@ -777,5 +812,5 @@ tree_store_closedir (tree_scan *scanner)
 {
 	g_assert (scanner != NULL);
 	
-	free (scanner);
+	g_free (scanner);
 }
