@@ -7,7 +7,7 @@
 /* String managing functions */
 
 #if defined(SCO_FLAVOR) && defined(__GNUC__)
-extern char* strdup(const char*);
+extern char*g_strdup (const char*);
 #endif
 
 int  is_printable (int c);
@@ -23,8 +23,6 @@ char *string_perm (mode_t mode_bits);
 char *strip_home_and_password(char *dir);
 char *extension (char *);
 char *split_extension (char *, int pad);
-char *get_full_name (char *dir, char *file);
-char *copy_strings (const char *first, ...);
 char *concat_dir_and_file (const char *dir, const char *file);
 char *unix_error_string (int error_num);
 char *skip_separators (char *s);
@@ -37,16 +35,18 @@ char *resolve_symlinks (char *path);
 char *diff_two_paths (char *first, char *second);
 int  set_nonblocking (int fd);
 
-#ifndef HAVE_STRCASECMP
-int strcasecmp (const char *s, const char *d);
-#endif
-
 char *x_basename (char *s);
 
 extern int align_extensions;
-#ifndef HAVE_MAD
-void *do_xmalloc (int);
-#define xmalloc(a,b) do_xmalloc (a)
+
+#ifdef HAVE_MAD
+char *mad_strconcat (const char *first, ...);
+char *mad_strdup_printf (const char *format, ...);
+char *mad_strdup_vprintf (const char *format, va_list args);
+
+#define g_strconcat		mad_strconcat
+#define g_strdup_printf		mad_strdup_printf
+#define g_strdup_vprintf	mad_strdup_vprintf
 #endif
 
 /* Profile managing functions */
